#!/usr/bin/env python


"""

	author: Shardul Nalegave
	license:

		MIT License

		Copyright (c) 2018 Shardul Nalegave

		Permission is hereby granted, free of charge, to any person obtaining a copy
		of this software and associated documentation files (the "Software"), to deal
		in the Software without restriction, including without limitation the rights
		to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
		copies of the Software, and to permit persons to whom the Software is
		furnished to do so, subject to the following conditions:

		The above copyright notice and this permission notice shall be included in all
		copies or substantial portions of the Software.

		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
		IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
		FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
		AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
		LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
		OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
		SOFTWARE.

"""



__version__ = '0.1.0'


import os
import sys
from sys import platform, stdout


class Signale:

	def __init__(self, opts={"scope": None, "underlined": False}):

		self.options = opts

		try:
			self.custom_loggers_conf = opts["custom"]
			for conf in self.custom_loggers_conf:
				func = lambda text="", prefix="", suffix="": self.log(text, prefix, suffix, conf)
				setattr(self, conf["name"], func)
		except KeyError:
			pass

		try:
			self.underlined = opts["underlined"]
		except KeyError:
			self.underlined = False

		try:
			scope = opts["scope"]
			if scope != None:
				self.scope = scope if scope != "" else "global"
			else:
				self.scope = None
		except KeyError:
			self.scope = None

		if platform == "win32":
			self.figures = {
				"pause": "||",
				"tick": '√',
				"cross": '×',
				"star": '*',
				"squareSmallFilled": '[█]',
				"play": '►',
				"bullet": '*',
				"ellipsis": '...',
				"pointerSmall": '»',
				"info": 'i',
				"warning": '‼',
				"heart": '♥',
				"radioOn": '(*)',
				"radioOff": '( )'
			}
		else:
			self.figures = {
				"pause": "||",
				"tick": '✔',
				"cross": '✖',
				"star": '★',
				"squareSmallFilled": '◼',
				"play": '▶',
				"bullet": '●',
				"ellipsis": '…',
				"pointerSmall": '›',
				"info": 'ℹ',
				"warning": '⚠',
				"heart": '♥',
				"radioOn": '◉',
				"radioOff": '◯'
			}

		self.colors = {
			"green": "\u001b[32;1m",
			"grey": "\u001b[38;5;240m",
			"red": "\u001b[38;5;196m",
			"yellow": "\u001b[38;5;11m",
			"purple": "\u001b[38;5;127m",
			"dark blue": "\u001b[38;5;33m",
			"cyan": "\u001b[36;1m",
			"blue": "\u001b[38;5;39m",
			"pink": "\u001b[38;5;198m",
			"reset": "\u001b[0m"
		}

		self.txt_decorations = {
			"bold": "\u001b[1m",
			"underline": "\u001b[4m",
			"reversed": "\u001b[7m"
		}

	def bold(self, text):
		bold = self.txt_decorations["bold"]
		reset = self.colors["reset"]
		return f"{bold}{text}{reset}"

	def underline(self, text):
		underline = self.txt_decorations["underline"]
		reset = self.colors["reset"]
		return f"{underline}{text}{reset}"

	def coloured(self, color, text):
		color = self.colors[color]
		reset = self.colors["reset"]
		return f"{color}{text}{reset}"

	def reversed(self, text):
		reversed = self.txt_decorations["reversed"]
		reset = self.colors["reset"]
		return f"{reversed}{text}{reset}"

	def logger_label(self, color, icon, label):
		if self.underlined == True:
			label = f"\u001b[4m{label}\u001b[0m"
		label = f"\u001b[1m{label}\u001b[0m"
		label = self.coloured(color, "{} {}".format(icon, label))
		return label

	def logger(self, text="", prefix="", suffix=""):
		message = ""
		if prefix != "":
			pointer = self.figures["pointerSmall"]
			message = f"  \u001b[38;5;248m[{prefix}] {pointer}\u001b[0m  {text}"
		else:
			message = f"  {text}"
		if suffix != "":
			message += f"   \u001b[38;5;245m-- {suffix}\u001b[0m"
		if self.scope != None:
			if isinstance(self.scope, list):
				message = "   " + message
				scopes = ""
				for item in self.scope:
					scopes += f" \u001b[38;5;248m[{item}]\u001b[0m"
				if prefix == "":
					pointer = self.figures["pointerSmall"]
					message = f"  \u001b[38;5;248m{scopes} \u001b[38;5;248m{pointer}\u001b[0m" + message
				else:
					message = f" \u001b[38;5;248m{scopes}\u001b[0m" + message
			else:
				if prefix == "":
					pointer = self.figures["pointerSmall"]
					message = f"   \u001b[38;5;248m[{self.scope}] \u001b[38;5;248m{pointer}\u001b[0m" + message
				else:
					message = f"   \u001b[38;5;248m[{self.scope}]\u001b[0m" + message
				# message = f"   \u001b[38;5;248m[{self.scope}]\u001b[0m" + message
		return message

	def log(self, text="", prefix="", suffix="", conf={}):
		text = "{}:  {}".format(self.logger_label(conf["color"], conf["badge"], "{}".format(conf["label"])), text)
		message = self.logger(text, prefix, suffix)
		print(message)

	def simple(self, text="", prefix="", suffix=""):
		print(self.logger(text, prefix, suffix))

	def success(self, text="", prefix="", suffix=""):
		tick = self.figures["tick"]
		text = "{}:  {}".format(self.logger_label("green", tick, "Success"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def start(self, text="", prefix="", suffix=""):
		icon = self.figures["play"]
		text = "{}:  {}".format(self.logger_label("green", icon, "Start"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def error(self, text="", prefix="", suffix=""):
		cross = self.figures["cross"]
		text = "{}:  {}".format(self.logger_label("red", cross, "Error"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def warning(self, text="", prefix="", suffix=""):
		icon = self.figures["warning"]
		text = "{}:  {}".format(self.logger_label("yellow", icon, "Warning"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def watch(self, text="", prefix="", suffix=""):
		icon = self.figures["ellipsis"]
		text = "{}:  {}".format(self.logger_label("yellow", icon, "Watching"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def stop(self, text="", prefix="", suffix=""):
		icon = self.figures["squareSmallFilled"]
		text = "{}:  {}".format(self.logger_label("red", icon, "Stop"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def important(self, text="", prefix="", suffix=""):
		icon = self.figures["star"]
		text = "{}:  {}".format(self.logger_label("yellow", icon, "Important"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def pending(self, text="", prefix="", suffix=""):
		icon = self.figures["radioOff"]
		text = "{}:  {}".format(self.logger_label("purple", icon, "Pending"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def debug(self, text="", prefix="", suffix=""):
		icon = self.figures["squareSmallFilled"]
		text = "{}:  {}".format(self.logger_label("dark blue", icon, "Debug"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def info(self, text="", prefix="", suffix=""):
		icon = self.figures["info"]
		text = "{}:  {}".format(self.logger_label("cyan", icon, "Info"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def pause(self, text="", prefix="", suffix=""):
		icon = self.figures["pause"]
		text = "{}:  {}".format(self.logger_label("yellow", icon, "Pause"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def complete(self, text="", prefix="", suffix=""):
		icon = self.figures["radioOn"]
		text = "{}:  {}".format(self.logger_label("blue", icon, "Complete"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def like(self, text="", prefix="", suffix=""):
		icon = self.figures["heart"]
		text = "{}:  {}".format(self.logger_label("pink", icon, "Like"), text)
		message = self.logger(text=text, prefix=prefix, suffix=suffix)
		print(message)

	def center(self, text="", prefix="", suffix=""):
		rows, cols = os.popen('stty size', 'r').read().split()
		rows, cols = int(rows), int(cols)
		text = "-" * 10 + "  " + text + "  " + "-" * 10
		len_text = len(text)
		message = " " * ((cols - len_text) // 2) + text + " " * ((cols - len_text) // 2)
		print(message)

	def scoped(self, scope):
		opts = self.options
		if self.scope != None:
			if isinstance(self.scope, list):
				opts["scope"] = opts["scope"].append(scope)
				return Signale(opts)
			else:
				opts["scope"] = [self.scope, scope]
				return Signale(opts)
			return Signale()
		else:
			opts["scope"] = scope
			return Signale(opts)
		return Signale()

	def ask(self, questions=[]):
		answers = {}
		for q in questions:
			qtype = ""
			qreq = ""
			qdef = ""
			try:
				qtype = q["type"]
			except KeyError:
				qtype = "input"
			try:
				qreq = q["required"]
			except KeyError:
				qreq = False
			try:
				qdef = q["default"]
			except KeyError:
				qdef = ""
			if qtype == "input":
				if qdef == "":
					ans = ""
					try:
						ans = input(self.coloured("yellow", "  ? {}: ".format(q["message"])) + self.colors["cyan"])
					except KeyboardInterrupt:
						stdout.write(self.colors["reset"])
						stdout.write("\n")
						stdout.flush()
						KeyboardInterrupt()
					except EOFError:
						stdout.write(self.colors["reset"])
						stdout.write("\n")
						stdout.flush()
					stdout.write(self.colors["reset"])
					stdout.flush()
					if ans == "" and qreq == True:
						while ans == "":
							ans = self.ask([q])
					if isinstance(ans, str):
						answers[q["name"]] = ans
					elif isinstance(ans, dict):
						while isinstance(ans, dict):
							ans = ans[q["name"]]
						answers[q["name"]] = ans
				else:
					ans = ""
					try:
						ans = input(self.coloured("yellow", "  ? {} ".format(q["message"])) + self.coloured("pink", "({})".format(qdef)) + ": " + self.colors["cyan"])
					except KeyboardInterrupt:
						stdout.write(self.colors["reset"])
						stdout.write("\n")
						stdout.flush()
						KeyboardInterrupt()
					except EOFError:
						stdout.write(self.colors["reset"])
						stdout.write("\n")
						stdout.flush()
					stdout.write(self.colors["reset"])
					stdout.flush()
					if ans == "":
						ans = qdef
					if isinstance(ans, str):
						answers[q["name"]] = ans
					elif isinstance(ans, dict):
						while isinstance(ans, dict):
							ans = ans[q["name"]]
						answers[q["name"]] = ans
		return answers



# s = Signale({
# 	"underlined": False
# })
# s.center("Testing Logger")
# s.simple("ABC", prefix="Debugger", suffix="xyz")
# s.info("Starting", prefix="Debugger")
# s.success("Started Successfully", prefix="Debugger", suffix="xyz")
# s.watch("Watching All Files", prefix="Debugger")
# s.error("Something Went Wrong", prefix="Debugger")
# s.warning("Deprecation Warning", prefix="Debugger")
# s.pending("Postponed", prefix="Debugger")
# s.debug("Found A Bug on L55", prefix="Debugger")
# s.start("Started New Process", prefix="Debugger")
# s.pause("Process Paused", prefix="Debugger")
# s.complete("Task Completed", prefix="Debugger")
# s.important("New Update Available. Please Update!", prefix="Debugger")
# s.like("I Love Signale", prefix="Debugger")
# s.stop("Stopping", prefix="Debugger")

# print("\n")

# logger = Signale({
# 	"scope": ""
# })
# logger.success("Started Successfully", prefix="Debugger")
# logger.warning("`a` function is deprecated", suffix="main.py")
# logger.complete("Run Complete")

# print("\n")

# logger = Signale({"scope": "custom"})
# logger.success("Started Successfully", prefix="Debugger")
# logger.warning("`a` function is deprecated", suffix="main.py")
# logger.complete("Run Complete")

# logger = Signale({
# 	"scope": "global scope",
# 	"custom": [
# 		{
# 			"badge": "!",
# 			"label": "Attention",
# 			"color": "red",
# 			"name": "attention"
# 		}
# 	],
# 	"underlined": True
# })

# logger2 = logger.scoped("inner")

# logger.attention("It Works!")
# logger2.attention("With Logger2")


logger = Signale()
ans = logger.ask([
	{
		"type": "input",
		"name": "username",
		"message": "Your Name",
		"default": "Shardul"
	}
])

print(logger.bold(logger.coloured("pink", ans)))

# print("\n\n")

# print(logger.bold("Bold Text"))
# print(logger.underline("Underlined"))
# print(logger.reversed("Reversed"))


# logger = Signale() # Option can be passed to the constructor
# logger.info("Signale.py is amazing", prefix="Logger")

# logger = Signale({
# 	"scope": "global scope",
# 	"custom": [
# 		{
# 			"badge": "!",
# 			"label": "Attention",
# 			"color": "red",
# 			"name": "attention"
# 		}
# 	],
# 	"underlined": False
# })

# logger.attention("It Works!")
# logger.scoped("inner").attention("Salute Signale.py")


# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['signalepy']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'signalepy',
    'version': '0.3.7',
    'description': 'Elegant Console Logger For Python Command Line Apps',
    'long_description': '\n<div align="center">\n\t<h1 align="center"><a href="https://signale.netlify.com">Signale.py</a></h1>\n\t<p align="center">Elegant Console Logger For Python Command-Line Apps</p>\n\t<br>\n\t<br>\n\t<img src="./imgs/main.png" alt="example" align="center">\n</div>\n\n<hr>\n\n\n\n## Installation\n**Signale.py** can be installed using pip.\n\n````bash\n\n    [sudo] pip install signalepy\n\n````\n\n\n\n## Usage\nPackage consists of a class `Signale`, it is the main constructor class. The object created has all the logger functions in it.\n\n\n### Using Loggers\nEach logger function takes in three arguments:-\n- `text`\n- `prefix` ( Optional )\n- `suffix` ( Optional )\n\nThey all are available in the logger object. To create one do this:-\n````python\n\nfrom signalepy import Signale\n\nlogger = Signale()\n\n````\n\nNow you can use the default loggers using this object like:-\n````python\n\n...\n\nlogger.success("Started Successfully", prefix="Debugger")\nlogger.warning("`a` function is deprecated", suffix="main.py")\nlogger.complete("Run Complete")\n\n...\n\n````\n\n\nThis will produce the following result:-\n\n<div align="center">\n\t<img align="center" src="./imgs/result.png">\n</div>\n\n<br><br>\n\n<details>\n\t<summary>View All Available Loggers</summary>\n\n- `simple`\n- `success`\n- `error`\n- `warning`\n- `start`\n- `stop`\n- `watch`\n- `important`\n- `pending`\n- `complete`\n- `debug`\n- `pause`\n- `info`\n- `like`\n- `center`\n- `bold`\n- `underline`\n- `reversed`\n- `coloured`\n\n</details>\n\n\n\n----------------------------------------------------------------------------------------------------------\n\n\n\n## Scoped Loggers\nTo create scoped loggers, define the `scope` field in the `options` argument of constructor like:-\n\n````python\n\nfrom signalepy import Signale\n\nlogger = Signale({\n\t"scope": "global scope"\n})\nlogger.success("Scoped Logger Works!")\n\n````\n\nThis will produce the following result:-\n\n<div align="center">\n\t<img src="./imgs/scope_str.png" align="center">\n</div>\n\n<br><br>\n\nYou also create multiple scopes by setting the `scope` field to a list of strings like:-\n\n````python\n\nfrom signalepy import Signale\n\nlogger = Signale({\n\t"scope": ["global scope", "inner scope"]\n})\nlogger.success("Scoped Logger Works!")\n\n````\n\nThis will produce the following result:-\n\n<div align="center">\n\t<img src="./imgs/scope_list.png" align="center">\n</div>\n\n<br><br><br>\n\nYou can also create scoped a logger which is just clone of other by using by using the `scoped` function like this:-\n\n````python\n\nfrom signalepy import Signale\n\nlogger = Signale({\n\t"scope": "global scope"\n})\n\nlogger2 = logger.scoped("inner")\nlogger2.like("I love Signale.py", prefix="Shardul Nalegave", suffix="Creator")\n\n````\n\nThis will produce the following result:-\n\n<div align="center">\n\t<img src="./imgs/scoped_func.png" align="center">\n</div>\n\n<br>\n\n**Note:-** All the configuration will be copied to the new logger just the scope will be extended.\n\n\n----------------------------------------------------------------------------------------------------------\n\n\n\n## Custom Loggers\nCustom loggers can be created by passing a array of custom logger configuration to the field `custom` in the options passed to the constructor. Even clone loggers inherit the parent\'s custom loggers. For example:-\n\n````python\n\nfrom signalepy import Signale\n\nlogger = Signale({\n\t"scope": "global scope",\n\t"custom": [\n\t\t{\n\t\t\t"badge": "!",\n\t\t\t"label": "Attention",\n\t\t\t"color": "red",\n\t\t\t"name": "attention"\n\t\t}\n\t]\n})\n\nlogger2 = logger.scoped("inner")\n\nlogger.attention("It Works!")\nlogger2.attention("With Logger2")\n\n````\n\nThis will produce the following result:-\n\n<div align="center">\n\t<img src="./imgs/custom.png" align="center">\n</div>\n\n----------------------------------------------------------------------------------------------------------\n\n\n\n## Underlined Labels\nYou can set labels to be underlined by setting the `underlined` field to `True` in the options passed to the constructor. For example:-\n\n````python\n\nfrom signalepy import Signale\n\ns = Signale({\n\t"underlined": True\n})\ns.center("Testing Logger")\ns.simple("ABC", prefix="Debugger", suffix="xyz")\ns.info("Starting", prefix="Debugger")\ns.success("Started Successfully", prefix="Debugger", suffix="xyz")\ns.watch("Watching All Files", prefix="Debugger")\ns.error("Something Went Wrong", prefix="Debugger")\ns.warning("Deprecation Warning", prefix="Debugger")\ns.pending("Postponed", prefix="Debugger")\ns.debug("Found A Bug on L55", prefix="Debugger")\ns.start("Started New Process", prefix="Debugger")\ns.pause("Process Paused", prefix="Debugger")\ns.complete("Task Completed", prefix="Debugger")\ns.important("New Update Available. Please Update!", prefix="Debugger")\ns.like("I Love Signale", prefix="Debugger")\ns.stop("Stopping", prefix="Debugger")\n\n````\n\nThis will produce the following result:-\n\n<div align="center">\n\t<img src="./imgs/underlined.png" align="center">\n</div>\n\n\n\n----------------------------------------------------------------------------------------------------------\n\n\n\n## Options\nOptions taken by constructor\n\n1. `scope`\n\n\t<br>\n\t\n\t- Type: `str` or `list`\n\n\tSignale Logger Scope\n\n\t<br><br>\n\n3. `underlined`\n\n\t<br>\n\n\t- Type: `bool`\n\n\tLabels Should Be Underlined Or Not\n\n2. `custom`\n\n\t<br>\n\n\t- Type: `list`\n\n\tList of custom logger configuration.\n\n\t- Configuration Type: `dict`\n\n\t\tCustom Logger Configuration Dictionary\n\n\t\t- Keys\n\n\t\t\t1. `name`\n\n\t\t\t\t- Type: `str`\n\n\t\t\t\tName of the logger\n\n\t\t\t2. `label`\n\n\t\t\t\t- Type: `str`\n\n\t\t\t\tLabel displayed beside the icon ( colored text )\n\n\t\t\t3. `badge`\n\n\t\t\t\t- Type: `str`\n\n\t\t\t\tIcon\n\n\t\t\t4. `color`\n\n\t\t\t\t- Type: `str`\n\n\t\t\t\tColor of the icon and label. Should be from:-\n\n\t\t\t\t- `red`\n\t\t\t\t- `dark`\n\t\t\t\t- `blue`\n\t\t\t\t- `pink`\n\t\t\t\t- `green`\n\t\t\t\t- `grey`\n\t\t\t\t- `purple`\n\t\t\t\t- `yellow`\n\t\t\t\t- `cyan`\n\t\t\t\t- `reset` (color reset code)\n\n\t<br><br>\n\n\n\n----------------------------------------------------------------------------------------------------------\n\n\n\n## API\n\n1. logger = `Signale(<options>)`\n\n\t<br>\n\n\t`Signale`\n\n\t- Type: `class`\n\n\tSignale class imported from `signalepy` module\n\n\t<br>\n\n\t`options`\n\n\t- Type: `dict`\n\n\tOptions Dictionary for logger.\n\n\t<br>\n\n\t- Returns: Signale Logger Object\n\n\tLogger object which can be used for logging\n\n\t<br><br>\n\n2. logger.`<logger>(message="", prefix="", suffix="")`\n\n\t<br>\n\n\t`logger`\n\n\t- Type: `function`\n\n\tCan be any default logger\n\n\t<br>\n\n\t`message`\n\n\t- Type: `str`\n\n\tMessage to be displayed\n\n\t<br>\n\n\t`prefix`\n\n\t- Type: `str`\n\t- Required: False\n\n\tPrefix text\n\n\t<br>\n\n\t`suffix`\n\n\t- Type: `str`\n\t- Required: False\n\n\tSuffix text\n\n\t<br>\n\n\t- Returns: `None`\n\n\t<br><br>\n\n3. logger2 = `logger`.scoped(`<new scope>`)\n\n\t<br>\n\n\t`logger`\n\n\t- Type: Signale Logger Object\n\n\tParent Logger\n\n\t<br>\n\n\t`new scope`\n\n\t- Type: `str`\n\n\tNew Scope Name\n\n\t<br>\n\n\t- Returns: Signale Logger Object\n\n\tClone Logger object with extended scope\n\n\t<br><br>\n\n\n\n----------------------------------------------------------------------------------------------------------\n\n\n\n**Licensed Under [MIT License](https://github.com/ShardulNalegave/signale.py/blob/master/LICENSE)**\n**A Project By [Shardul Nalegave](https://shardul.netlify.com)**\n',
    'author': 'Shardul Nalegave',
    'author_email': 'nalegaveshardul40@gmail.com',
    'url': 'https://github.com/ShardulNalegave/signale.py',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.0,<4.0',
}


setup(**setup_kwargs)
